package ca.odell.glazedlists;

import junit.framework.*;
import ca.odell.glazedlists.event.*;

/**
 * Validates that {@link EventList}s can recover from {@link RuntimeException}s.
 *
 * @author <a href="mailto:jesse@odel.on.ca">Jesse Wilson</a>
 */
public class RuntimeExceptionTest extends TestCase {
    
    public RuntimeExceptionTest() {
        super();
    }
    
    /**
     * the exception fired by the event lists 
     */
    private RuntimeException luckyException = new RuntimeException();
    
    /**
     * list to fire events 
     */
    private BasicEventList source = new BasicEventList();
    
    /**
     * listener that throws exceptions 
     */
    private ExceptionThrower exceptionThrower = new ExceptionThrower();
    
    /**
     * listener that validates events are received 
     */
    private ListConsistencyListener listConsistencyListener = new ListConsistencyListener(source, "exceptions");
    
    /**
     * Verifies that an Exception thrown by a ListEventListener is rethrown.
     */
    public void testExceptionRethrown() {
        source.addListEventListener(exceptionThrower);
        source.addListEventListener(listConsistencyListener);
        source.add("Dan");
        source.add("Frank");
        source.add("Larry");
        listConsistencyListener.assertConsistent();
        exceptionThrower.setNextException(luckyException);
        try {
            source.add("Adam");
            fail("this statement shouldn\'t be reached");
        } catch (RuntimeException e) {
            assertTrue(e == luckyException);
            listConsistencyListener.assertConsistent();
        }
    }
    
    /**
     * Verifies that an Exception thrown by a ListEventListener is rethrown.
     */
    public void testExceptionRethrownListenerSecond() {
        source.addListEventListener(listConsistencyListener);
        source.addListEventListener(exceptionThrower);
        source.add("Matt");
        source.add("Kevin");
        source.add("Julie");
        listConsistencyListener.assertConsistent();
        exceptionThrower.setNextException(luckyException);
        try {
            source.add("Leanne");
            fail("this statement shouldn\'t be reached");
        } catch (RuntimeException e) {
            assertTrue(e == luckyException);
            listConsistencyListener.assertConsistent();
        }
    }
    
    /**
     * Verifies that an Exception thrown by a ListEventListener is rethrown.
     */
    public void testMultipleExceptions() {
        source.addListEventListener(exceptionThrower);
        source.addListEventListener(listConsistencyListener);
        ExceptionThrower exceptionThrower2 = new ExceptionThrower();
        source.addListEventListener(exceptionThrower2);
        source.add("Leanne");
        source.add("Bev");
        source.add("Jesse");
        listConsistencyListener.assertConsistent();
        exceptionThrower.setNextException(luckyException);
        exceptionThrower2.setNextException(new NullPointerException());
        try {
            source.add("Eric");
            fail("this statement shouldn\'t be reached");
        } catch (RuntimeException e) {
            assertTrue(e == luckyException);
            listConsistencyListener.assertConsistent();
        }
    }
    
    /**
     * ListEventListener that throws an exception on demand.
     */
    class ExceptionThrower implements ListEventListener {
        
        ExceptionThrower() {
            super();
        }
        private RuntimeException nextException = null;
        
        public void setNextException(RuntimeException nextException) {
            this.nextException = nextException;
        }
        
        public void listChanged(ListEvent listChanges) {
            if (nextException == null) return;
            RuntimeException toThrow = nextException;
            nextException = null;
            throw toThrow;
        }
    }
}
